@@ -12,6 +12,20 @@
 #define _MM_CLAMP_SS(r,min,max)				\
 	_mm_min_ss(_mm_max_ss(r, min), max)
 
+#define _MM_BSWAP_EPI16(x)						\
+({									\
+	_mm_or_si128(							\
+		_mm_slli_epi16(x, 8),					\
+		_mm_srli_epi16(x, 8));					\
+})
+
+#define _MM_BSWAP_EPI32(x)						\
+({									\
+	__m128i a = _MM_BSWAP_EPI16(x);					\
+	a = _mm_shufflelo_epi16(a, _MM_SHUFFLE(2, 3, 0, 1));		\
+	a = _mm_shufflehi_epi16(a, _MM_SHUFFLE(2, 3, 0, 1));		\
+})
+
 static void
 conv_s16_to_f32d_1s_sse2(void *data, void * SPA_RESTRICT dst[], const void * SPA_RESTRICT src,
 		uint32_t n_channels, uint32_t n_samples)
@@ -57,6 +71,52 @@ conv_s16_to_f32d_sse2(struct convert *conv, void * SPA_RESTRICT dst[], const voi
 		conv_s16_to_f32d_1s_sse2(conv, &dst[i], &s[i], n_channels, n_samples);
 }
 
+static void
+conv_s16s_to_f32d_1s_sse2(void *data, void * SPA_RESTRICT dst[], const void * SPA_RESTRICT src,
+		uint32_t n_channels, uint32_t n_samples)
+{
+	const uint16_t *s = src;
+	float *d0 = dst[0];
+	uint32_t n, unrolled;
+	__m128i in = _mm_setzero_si128();
+	__m128 out, factor = _mm_set1_ps(1.0f / S16_SCALE);
+
+	if (SPA_LIKELY(SPA_IS_ALIGNED(d0, 16)))
+		unrolled = n_samples & ~3;
+	else
+		unrolled = 0;
+
+	for(n = 0; n < unrolled; n += 4) {
+		in = _mm_insert_epi16(in, s[0*n_channels], 1);
+		in = _mm_insert_epi16(in, s[1*n_channels], 3);
+		in = _mm_insert_epi16(in, s[2*n_channels], 5);
+		in = _mm_insert_epi16(in, s[3*n_channels], 7);
+		in = _MM_BSWAP_EPI16(in);
+		in = _mm_srai_epi32(in, 16);
+		out = _mm_cvtepi32_ps(in);
+		out = _mm_mul_ps(out, factor);
+		_mm_store_ps(&d0[n], out);
+		s += 4*n_channels;
+	}
+	for(; n < n_samples; n++) {
+		out = _mm_cvtsi32_ss(factor, (int16_t)bswap_16(s[0]));
+		out = _mm_mul_ss(out, factor);
+		_mm_store_ss(&d0[n], out);
+		s += n_channels;
+	}
+}
+
+void
+conv_s16s_to_f32d_sse2(struct convert *conv, void * SPA_RESTRICT dst[], const void * SPA_RESTRICT src[],
+		uint32_t n_samples)
+{
+	const uint16_t *s = src[0];
+	uint32_t i = 0, n_channels = conv->n_channels;
+
+	for(; i < n_channels; i++)
+		conv_s16s_to_f32d_1s_sse2(conv, &dst[i], &s[i], n_channels, n_samples);
+}
+
 void
 conv_s16_to_f32d_2_sse2(struct convert *conv, void * SPA_RESTRICT dst[], const void * SPA_RESTRICT src[],
 		uint32_t n_samples)
@@ -114,6 +174,65 @@ conv_s16_to_f32d_2_sse2(struct convert *conv, void * SPA_RESTRICT dst[], const v
 	}
 }
 
+void
+conv_s16s_to_f32d_2_sse2(struct convert *conv, void * SPA_RESTRICT dst[], const void * SPA_RESTRICT src[],
+		uint32_t n_samples)
+{
+	const uint16_t *s = src[0];
+	float *d0 = dst[0], *d1 = dst[1];
+	uint32_t n, unrolled;
+	__m128i in[2], t[4];
+	__m128 out[4], factor = _mm_set1_ps(1.0f / S16_SCALE);
+
+	if (SPA_IS_ALIGNED(s, 16) &&
+	    SPA_IS_ALIGNED(d0, 16) &&
+	    SPA_IS_ALIGNED(d1, 16))
+		unrolled = n_samples & ~7;
+	else
+		unrolled = 0;
+
+	for(n = 0; n < unrolled; n += 8) {
+		in[0] = _mm_load_si128((__m128i*)(s + 0));
+		in[1] = _mm_load_si128((__m128i*)(s + 8));
+		in[0] = _MM_BSWAP_EPI16(in[0]);
+		in[1] = _MM_BSWAP_EPI16(in[1]);
+
+		t[0] = _mm_slli_epi32(in[0], 16);
+		t[0] = _mm_srai_epi32(t[0], 16);
+		out[0] = _mm_cvtepi32_ps(t[0]);
+		out[0] = _mm_mul_ps(out[0], factor);
+
+		t[1] = _mm_srai_epi32(in[0], 16);
+		out[1] = _mm_cvtepi32_ps(t[1]);
+		out[1] = _mm_mul_ps(out[1], factor);
+
+		t[2] = _mm_slli_epi32(in[1], 16);
+		t[2] = _mm_srai_epi32(t[2], 16);
+		out[2] = _mm_cvtepi32_ps(t[2]);
+		out[2] = _mm_mul_ps(out[2], factor);
+
+		t[3] = _mm_srai_epi32(in[1], 16);
+		out[3] = _mm_cvtepi32_ps(t[3]);
+		out[3] = _mm_mul_ps(out[3], factor);
+
+		_mm_store_ps(&d0[n + 0], out[0]);
+		_mm_store_ps(&d1[n + 0], out[1]);
+		_mm_store_ps(&d0[n + 4], out[2]);
+		_mm_store_ps(&d1[n + 4], out[3]);
+
+		s += 16;
+	}
+	for(; n < n_samples; n++) {
+		out[0] = _mm_cvtsi32_ss(factor, (int16_t)bswap_16(s[0]));
+		out[0] = _mm_mul_ss(out[0], factor);
+		out[1] = _mm_cvtsi32_ss(factor, (int16_t)bswap_16(s[1]));
+		out[1] = _mm_mul_ss(out[1], factor);
+		_mm_store_ss(&d0[n], out[0]);
+		_mm_store_ss(&d1[n], out[1]);
+		s += 2;
+	}
+}
+
 #define spa_read_unaligned(ptr, type) \
 __extension__ ({ \
 	__typeof__(type) _val; \
@@ -335,7 +454,7 @@ conv_s32_to_f32d_1s_sse2(void *data, void * SPA_RESTRICT dst[], const void * SPA
 	float *d0 = dst[0];
 	uint32_t n, unrolled;
 	__m128i in;
-	__m128 out, factor = _mm_set1_ps(1.0f / S24_SCALE);
+	__m128 out, factor = _mm_set1_ps(1.0f / S32_SCALE_I2F);
 
 	if (SPA_IS_ALIGNED(d0, 16))
 		unrolled = n_samples & ~3;
@@ -347,14 +466,13 @@ conv_s32_to_f32d_1s_sse2(void *data, void * SPA_RESTRICT dst[], const void * SPA
 				    s[1*n_channels],
 				    s[2*n_channels],
 				    s[3*n_channels]);
-		in = _mm_srai_epi32(in, 8);
 		out = _mm_cvtepi32_ps(in);
 		out = _mm_mul_ps(out, factor);
 		_mm_store_ps(&d0[n], out);
 		s += 4*n_channels;
 	}
 	for(; n < n_samples; n++) {
-		out = _mm_cvtsi32_ss(factor, s[0]>>8);
+		out = _mm_cvtsi32_ss(factor, s[0]);
 		out = _mm_mul_ss(out, factor);
 		_mm_store_ss(&d0[n], out);
 		s += n_channels;
@@ -381,9 +499,9 @@ conv_f32d_to_s32_1s_sse2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 	uint32_t n, unrolled;
 	__m128 in[1];
 	__m128i out[4];
-	__m128 scale = _mm_set1_ps(S24_SCALE);
-	__m128 int_min = _mm_set1_ps(S24_MIN);
-	__m128 int_max = _mm_set1_ps(S24_MAX);
+	__m128 scale = _mm_set1_ps(S32_SCALE_F2I);
+	__m128 int_min = _mm_set1_ps(S32_MIN_F2I);
+	__m128 int_max = _mm_set1_ps(S32_MAX_F2I);
 
 	if (SPA_IS_ALIGNED(s0, 16))
 		unrolled = n_samples & ~3;
@@ -394,7 +512,6 @@ conv_f32d_to_s32_1s_sse2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 		in[0] = _mm_mul_ps(_mm_load_ps(&s0[n]), scale);
 		in[0] = _MM_CLAMP_PS(in[0], int_min, int_max);
 		out[0] = _mm_cvtps_epi32(in[0]);
-		out[0] = _mm_slli_epi32(out[0], 8);
 		out[1] = _mm_shuffle_epi32(out[0], _MM_SHUFFLE(0, 3, 2, 1));
 		out[2] = _mm_shuffle_epi32(out[0], _MM_SHUFFLE(1, 0, 3, 2));
 		out[3] = _mm_shuffle_epi32(out[0], _MM_SHUFFLE(2, 1, 0, 3));
@@ -409,7 +526,7 @@ conv_f32d_to_s32_1s_sse2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 		in[0] = _mm_load_ss(&s0[n]);
 		in[0] = _mm_mul_ss(in[0], scale);
 		in[0] = _MM_CLAMP_SS(in[0], int_min, int_max);
-		*d = _mm_cvtss_si32(in[0]) << 8;
+		*d = _mm_cvtss_si32(in[0]);
 		d += n_channels;
 	}
 }
@@ -423,12 +540,12 @@ conv_f32d_to_s32_2s_sse2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 	uint32_t n, unrolled;
 	__m128 in[2];
 	__m128i out[2], t[2];
-	__m128 scale = _mm_set1_ps(S24_SCALE);
-	__m128 int_min = _mm_set1_ps(S24_MIN);
-	__m128 int_max = _mm_set1_ps(S24_MAX);
+	__m128 scale = _mm_set1_ps(S32_SCALE_F2I);
+	__m128 int_min = _mm_set1_ps(S32_MIN_F2I);
+	__m128 int_max = _mm_set1_ps(S32_MAX_F2I);
 
 	if (SPA_IS_ALIGNED(s0, 16) &&
-	    SPA_IS_ALIGNED(s1, 16))
+		SPA_IS_ALIGNED(s1, 16))
 		unrolled = n_samples & ~3;
 	else
 		unrolled = 0;
@@ -442,8 +559,6 @@ conv_f32d_to_s32_2s_sse2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 
 		out[0] = _mm_cvtps_epi32(in[0]);
 		out[1] = _mm_cvtps_epi32(in[1]);
-		out[0] = _mm_slli_epi32(out[0], 8);
-		out[1] = _mm_slli_epi32(out[1], 8);
 
 		t[0] = _mm_unpacklo_epi32(out[0], out[1]);
 		t[1] = _mm_unpackhi_epi32(out[0], out[1]);
@@ -463,7 +578,6 @@ conv_f32d_to_s32_2s_sse2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 		in[0] = _mm_mul_ps(in[0], scale);
 		in[0] = _MM_CLAMP_PS(in[0], int_min, int_max);
 		out[0] = _mm_cvtps_epi32(in[0]);
-		out[0] = _mm_slli_epi32(out[0], 8);
 		_mm_storel_epi64((__m128i*)d, out[0]);
 		d += n_channels;
 	}
@@ -478,14 +592,14 @@ conv_f32d_to_s32_4s_sse2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 	uint32_t n, unrolled;
 	__m128 in[4];
 	__m128i out[4];
-	__m128 scale = _mm_set1_ps(S24_SCALE);
-	__m128 int_min = _mm_set1_ps(S24_MIN);
-	__m128 int_max = _mm_set1_ps(S24_MAX);
+	__m128 scale = _mm_set1_ps(S32_SCALE_F2I);
+	__m128 int_min = _mm_set1_ps(S32_MIN_F2I);
+	__m128 int_max = _mm_set1_ps(S32_MAX_F2I);
 
 	if (SPA_IS_ALIGNED(s0, 16) &&
-	    SPA_IS_ALIGNED(s1, 16) &&
-	    SPA_IS_ALIGNED(s2, 16) &&
-	    SPA_IS_ALIGNED(s3, 16))
+		SPA_IS_ALIGNED(s1, 16) &&
+		SPA_IS_ALIGNED(s2, 16) &&
+		SPA_IS_ALIGNED(s3, 16))
 		unrolled = n_samples & ~3;
 	else
 		unrolled = 0;
@@ -507,10 +621,6 @@ conv_f32d_to_s32_4s_sse2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 		out[1] = _mm_cvtps_epi32(in[1]);
 		out[2] = _mm_cvtps_epi32(in[2]);
 		out[3] = _mm_cvtps_epi32(in[3]);
-		out[0] = _mm_slli_epi32(out[0], 8);
-		out[1] = _mm_slli_epi32(out[1], 8);
-		out[2] = _mm_slli_epi32(out[2], 8);
-		out[3] = _mm_slli_epi32(out[3], 8);
 
 		_mm_storeu_si128((__m128i*)(d + 0*n_channels), out[0]);
 		_mm_storeu_si128((__m128i*)(d + 1*n_channels), out[1]);
@@ -531,7 +641,6 @@ conv_f32d_to_s32_4s_sse2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 		in[0] = _mm_mul_ps(in[0], scale);
 		in[0] = _MM_CLAMP_PS(in[0], int_min, int_max);
 		out[0] = _mm_cvtps_epi32(in[0]);
-		out[0] = _mm_slli_epi32(out[0], 8);
 		_mm_storeu_si128((__m128i*)d, out[0]);
 		d += n_channels;
 	}
@@ -620,6 +729,7 @@ void conv_noise_tri_hf_sse2(struct convert *conv, float *noise, uint32_t n_sampl
 	_mm_store_si128((__m128i*)p, old[0]);
 }
 
+// FIXME: this function is not covered with tests.
 static void
 conv_f32d_to_s32_1s_noise_sse2(struct convert *conv, void * SPA_RESTRICT dst, const void * SPA_RESTRICT src,
 		float *noise, uint32_t n_channels, uint32_t n_samples)
@@ -629,9 +739,9 @@ conv_f32d_to_s32_1s_noise_sse2(struct convert *conv, void * SPA_RESTRICT dst, co
 	uint32_t n, unrolled;
 	__m128 in[1];
 	__m128i out[4];
-	__m128 scale = _mm_set1_ps(S24_SCALE);
-	__m128 int_min = _mm_set1_ps(S24_MIN);
-	__m128 int_max = _mm_set1_ps(S24_MAX);
+	__m128 scale = _mm_set1_ps(S32_SCALE_F2I);
+	__m128 int_min = _mm_set1_ps(S32_MIN_F2I);
+	__m128 int_max = _mm_set1_ps(S32_MAX_F2I);
 
 	if (SPA_IS_ALIGNED(s, 16))
 		unrolled = n_samples & ~3;
@@ -643,7 +753,6 @@ conv_f32d_to_s32_1s_noise_sse2(struct convert *conv, void * SPA_RESTRICT dst, co
 		in[0] = _mm_add_ps(in[0], _mm_load_ps(&noise[n]));
 		in[0] = _MM_CLAMP_PS(in[0], int_min, int_max);
 		out[0] = _mm_cvtps_epi32(in[0]);
-		out[0] = _mm_slli_epi32(out[0], 8);
 		out[1] = _mm_shuffle_epi32(out[0], _MM_SHUFFLE(0, 3, 2, 1));
 		out[2] = _mm_shuffle_epi32(out[0], _MM_SHUFFLE(1, 0, 3, 2));
 		out[3] = _mm_shuffle_epi32(out[0], _MM_SHUFFLE(2, 1, 0, 3));
@@ -659,7 +768,7 @@ conv_f32d_to_s32_1s_noise_sse2(struct convert *conv, void * SPA_RESTRICT dst, co
 		in[0] = _mm_mul_ss(in[0], scale);
 		in[0] = _mm_add_ss(in[0], _mm_load_ss(&noise[n]));
 		in[0] = _MM_CLAMP_SS(in[0], int_min, int_max);
-		*d = _mm_cvtss_si32(in[0]) << 8;
+		*d = _mm_cvtss_si32(in[0]);
 		d += n_channels;
 	}
 }
@@ -766,15 +875,6 @@ conv_32d_to_32_sse2(struct convert *conv, void * SPA_RESTRICT dst[], const void
 		conv_interleave_32_1s_sse2(conv, &d[i], &src[i], n_channels, n_samples);
 }
 
-#define _MM_BSWAP_EPI32(x)						\
-({									\
-	__m128i a = _mm_or_si128(					\
-		_mm_slli_epi16(x, 8),					\
-		_mm_srli_epi16(x, 8));					\
-	a = _mm_shufflelo_epi16(a, _MM_SHUFFLE(2, 3, 0, 1));		\
-	a = _mm_shufflehi_epi16(a, _MM_SHUFFLE(2, 3, 0, 1));		\
-})
-
 static void
 conv_interleave_32s_1s_sse2(void *data, void * SPA_RESTRICT dst, const void * SPA_RESTRICT src[],
 		uint32_t n_channels, uint32_t n_samples)
@@ -1256,6 +1356,62 @@ conv_f32d_to_s16_sse2(struct convert *conv, void * SPA_RESTRICT dst[], const voi
 		conv_f32d_to_s16_1s_sse2(conv, &d[i], &src[i], n_channels, n_samples);
 }
 
+
+static void
+conv_f32d_to_s16s_1s_sse2(void *data, void * SPA_RESTRICT dst, const void * SPA_RESTRICT src[],
+		uint32_t n_channels, uint32_t n_samples)
+{
+	const float *s0 = src[0];
+	uint16_t *d = dst;
+	uint32_t n, unrolled;
+	__m128 in[2];
+	__m128i out[2];
+	__m128 int_scale = _mm_set1_ps(S16_SCALE);
+	__m128 int_max = _mm_set1_ps(S16_MAX);
+        __m128 int_min = _mm_set1_ps(S16_MIN);
+
+	if (SPA_IS_ALIGNED(s0, 16))
+		unrolled = n_samples & ~7;
+	else
+		unrolled = 0;
+
+	for(n = 0; n < unrolled; n += 8) {
+		in[0] = _mm_mul_ps(_mm_load_ps(&s0[n]), int_scale);
+		in[1] = _mm_mul_ps(_mm_load_ps(&s0[n+4]), int_scale);
+		out[0] = _mm_cvtps_epi32(in[0]);
+		out[1] = _mm_cvtps_epi32(in[1]);
+		out[0] = _mm_packs_epi32(out[0], out[1]);
+		out[0] = _MM_BSWAP_EPI16(out[0]);
+
+		d[0*n_channels] = _mm_extract_epi16(out[0], 0);
+		d[1*n_channels] = _mm_extract_epi16(out[0], 1);
+		d[2*n_channels] = _mm_extract_epi16(out[0], 2);
+		d[3*n_channels] = _mm_extract_epi16(out[0], 3);
+		d[4*n_channels] = _mm_extract_epi16(out[0], 4);
+		d[5*n_channels] = _mm_extract_epi16(out[0], 5);
+		d[6*n_channels] = _mm_extract_epi16(out[0], 6);
+		d[7*n_channels] = _mm_extract_epi16(out[0], 7);
+		d += 8*n_channels;
+	}
+	for(; n < n_samples; n++) {
+		in[0] = _mm_mul_ss(_mm_load_ss(&s0[n]), int_scale);
+		in[0] = _MM_CLAMP_SS(in[0], int_min, int_max);
+		*d = bswap_16((uint16_t)_mm_cvtss_si32(in[0]));
+		d += n_channels;
+	}
+}
+
+void
+conv_f32d_to_s16s_sse2(struct convert *conv, void * SPA_RESTRICT dst[], const void * SPA_RESTRICT src[],
+		uint32_t n_samples)
+{
+	uint16_t *d = dst[0];
+	uint32_t i = 0, n_channels = conv->n_channels;
+
+	for(; i < n_channels; i++)
+		conv_f32d_to_s16s_1s_sse2(conv, &d[i], &src[i], n_channels, n_samples);
+}
+
 static void
 conv_f32d_to_s16_1s_noise_sse2(struct convert *conv, void * SPA_RESTRICT dst, const void * SPA_RESTRICT src,
 		const float *noise, uint32_t n_channels, uint32_t n_samples)
@@ -1428,3 +1584,58 @@ conv_f32d_to_s16_2_sse2(struct convert *conv, void * SPA_RESTRICT dst[], const v
 		d += 2;
 	}
 }
+
+void
+conv_f32d_to_s16s_2_sse2(struct convert *conv, void * SPA_RESTRICT dst[], const void * SPA_RESTRICT src[],
+		uint32_t n_samples)
+{
+	const float *s0 = src[0], *s1 = src[1];
+	uint16_t *d = dst[0];
+	uint32_t n, unrolled;
+	__m128 in[4];
+	__m128i out[4];
+	__m128 int_scale = _mm_set1_ps(S16_SCALE);
+	__m128 int_max = _mm_set1_ps(S16_MAX);
+        __m128 int_min = _mm_set1_ps(S16_MIN);
+
+	if (SPA_IS_ALIGNED(s0, 16) &&
+	    SPA_IS_ALIGNED(s1, 16))
+		unrolled = n_samples & ~7;
+	else
+		unrolled = 0;
+
+	for(n = 0; n < unrolled; n += 8) {
+		in[0] = _mm_mul_ps(_mm_load_ps(&s0[n+0]), int_scale);
+		in[1] = _mm_mul_ps(_mm_load_ps(&s1[n+0]), int_scale);
+		in[2] = _mm_mul_ps(_mm_load_ps(&s0[n+4]), int_scale);
+		in[3] = _mm_mul_ps(_mm_load_ps(&s1[n+4]), int_scale);
+
+		out[0] = _mm_cvtps_epi32(in[0]);
+		out[1] = _mm_cvtps_epi32(in[1]);
+		out[2] = _mm_cvtps_epi32(in[2]);
+		out[3] = _mm_cvtps_epi32(in[3]);
+
+		out[0] = _mm_packs_epi32(out[0], out[2]);
+		out[1] = _mm_packs_epi32(out[1], out[3]);
+
+		out[2] = _mm_unpacklo_epi16(out[0], out[1]);
+		out[3] = _mm_unpackhi_epi16(out[0], out[1]);
+
+		out[2] = _MM_BSWAP_EPI16(out[2]);
+		out[3] = _MM_BSWAP_EPI16(out[3]);
+
+		_mm_storeu_si128((__m128i*)(d+0), out[2]);
+		_mm_storeu_si128((__m128i*)(d+8), out[3]);
+
+		d += 16;
+	}
+	for(; n < n_samples; n++) {
+		in[0] = _mm_mul_ss(_mm_load_ss(&s0[n]), int_scale);
+		in[1] = _mm_mul_ss(_mm_load_ss(&s1[n]), int_scale);
+		in[0] = _MM_CLAMP_SS(in[0], int_min, int_max);
+		in[1] = _MM_CLAMP_SS(in[1], int_min, int_max);
+		d[0] = bswap_16((uint16_t)_mm_cvtss_si32(in[0]));
+		d[1] = bswap_16((uint16_t)_mm_cvtss_si32(in[1]));
+		d += 2;
+	}
+}
