@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2018 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <limits.h>
 #include <signal.h>
@@ -29,6 +9,9 @@
 #include <locale.h>
 
 #include <spa/utils/result.h>
+#include <spa/utils/cleanup.h>
+#include <spa/debug/file.h>
+
 #include <pipewire/pipewire.h>
 
 #include <pipewire/i18n.h>
@@ -45,8 +28,10 @@ static void show_help(const char *name, const char *config_name)
 {
 	fprintf(stdout, _("%s [options]\n"
 		"  -h, --help                            Show this help\n"
+		"  -v, --verbose                         Increase verbosity by one level\n"
 		"      --version                         Show version\n"
-		"  -c, --config                          Load config (Default %s)\n"),
+		"  -c, --config                          Load config (Default %s)\n"
+		"  -P  --properties                      Set context properties\n"),
 		name,
 		config_name);
 }
@@ -61,13 +46,15 @@ int main(int argc, char *argv[])
 		{ "version",	no_argument,		NULL, 'V' },
 		{ "config",	required_argument,	NULL, 'c' },
 		{ "verbose",	no_argument,		NULL, 'v' },
+		{ "properties",	required_argument,	NULL, 'P' },
 
 		{ NULL, 0, NULL, 0}
 	};
 	int c, res = 0;
 	char path[PATH_MAX];
 	const char *config_name;
-	enum spa_log_level level = pw_log_level;
+	enum spa_log_level level;
+	struct spa_error_location loc;
 
 	if (setenv("PIPEWIRE_INTERNAL", "1", 1) < 0)
 		fprintf(stderr, "can't set PIPEWIRE_INTERNAL env: %m");
@@ -78,7 +65,13 @@ int main(int argc, char *argv[])
 	setlocale(LC_ALL, "");
 	pw_init(&argc, &argv);
 
-	while ((c = getopt_long(argc, argv, "hVc:v", long_options, NULL)) != -1) {
+	level = pw_log_level;
+
+	properties = pw_properties_new(
+				PW_KEY_CONFIG_NAME, config_name,
+				NULL);
+
+	while ((c = getopt_long(argc, argv, "hVc:vP:", long_options, NULL)) != -1) {
 		switch (c) {
 		case 'v':
 			if (level < SPA_LOG_LEVEL_TRACE)
@@ -97,17 +90,24 @@ int main(int argc, char *argv[])
 			return 0;
 		case 'c':
 			config_name = optarg;
+			pw_properties_set(properties, PW_KEY_CONFIG_NAME, config_name);
 			break;
+
+		case 'P':
+			if (pw_properties_update_string_checked(properties, optarg, strlen(optarg), &loc) < 0) {
+				spa_debug_file_error_location(stderr, &loc,
+						"error: syntax error in --properties: %s",
+						loc.reason);
+				goto done;
+			}
+			break;
+
 		default:
 			res = -EINVAL;
 			goto done;
 		}
 	}
 
-	properties = pw_properties_new(
-				PW_KEY_CONFIG_NAME, config_name,
-				NULL);
-
 	loop = pw_main_loop_new(&properties->dict);
 	if (loop == NULL) {
 		pw_log_error("failed to create main-loop: %m");
@@ -118,8 +118,7 @@ int main(int argc, char *argv[])
 	pw_loop_add_signal(pw_main_loop_get_loop(loop), SIGINT, do_quit, loop);
 	pw_loop_add_signal(pw_main_loop_get_loop(loop), SIGTERM, do_quit, loop);
 
-	context = pw_context_new(pw_main_loop_get_loop(loop), properties, 0);
-	properties = NULL;
+	context = pw_context_new(pw_main_loop_get_loop(loop), spa_steal_ptr(properties), 0);
 
 	if (context == NULL) {
 		pw_log_error("failed to create context: %m");
