@@ -1,27 +1,7 @@
-/* PipeWire
- *
- * Copyright © 2021 Wim Taymans <wim.taymans@gmail.com>
- * Copyright © 2021 Arun Raghavan <arun@asymptotic.io>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2021 Wim Taymans <wim.taymans@gmail.com> */
+/* SPDX-FileCopyrightText: Copyright © 2021 Arun Raghavan <arun@asymptotic.io> */
+/* SPDX-License-Identifier: MIT */
 
 #include <spa/param/audio/format-utils.h>
 #include <spa/utils/hook.h>
@@ -31,6 +11,34 @@
 #include "../defs.h"
 #include "../module.h"
 
+/** \page page_pulse_module_loopback Loopback
+ *
+ * ## Module Name
+ *
+ * `module-loopback`
+ *
+ * ## Module Options
+ *
+ * @pulse_module_options@
+ *
+ * ## See Also
+ *
+ * \ref page_module_loopback "libpipewire-module-loopback"
+ */
+
+static const char *const pulse_module_options =
+	"source=<source to connect to> "
+	"sink=<sink to connect to> "
+	"latency_msec=<latency in ms> "
+	"rate=<sample rate> "
+	"channels=<number of channels> "
+	"channel_map=<channel map> "
+	"sink_input_properties=<proplist> "
+	"source_output_properties=<proplist> "
+	"source_dont_move=<boolean> "
+	"sink_dont_move=<boolean> "
+	"remix=<remix channels?> ";
+
 #define NAME "loopback"
 
 PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
@@ -42,11 +50,9 @@ struct module_loopback_data {
 	struct pw_impl_module *mod;
 	struct spa_hook mod_listener;
 
+	struct pw_properties *global_props;
 	struct pw_properties *capture_props;
 	struct pw_properties *playback_props;
-
-	struct spa_audio_info_raw info;
-	uint32_t latency_msec;
 };
 
 static void module_destroy(void *data)
@@ -67,8 +73,7 @@ static int module_loopback_load(struct module *module)
 	struct module_loopback_data *data = module->user_data;
 	FILE *f;
 	char *args;
-	size_t size, i;
-	char val[256];
+	size_t size;
 
 	pw_properties_setf(data->capture_props, PW_KEY_NODE_GROUP, "loopback-%u", module->index);
 	pw_properties_setf(data->playback_props, PW_KEY_NODE_GROUP, "loopback-%u", module->index);
@@ -79,20 +84,7 @@ static int module_loopback_load(struct module *module)
 		return -errno;
 
 	fprintf(f, "{");
-	if (data->info.channels != 0) {
-		fprintf(f, " audio.channels = %u", data->info.channels);
-		if (!(data->info.flags & SPA_AUDIO_FLAG_UNPOSITIONED)) {
-			fprintf(f, " audio.position = [ ");
-			for (i = 0; i < data->info.channels; i++)
-				fprintf(f, "%s%s", i == 0 ? "" : ",",
-					channel_id2name(data->info.position[i]));
-			fprintf(f, " ]");
-		}
-	}
-	if (data->latency_msec != 0)
-		fprintf(f, " target.delay.sec = %s",
-				spa_json_format_float(val, sizeof(val),
-					data->latency_msec / 1000.0f));
+	pw_properties_serialize_dict(f, &data->global_props->dict, 0);
 	fprintf(f, " capture.props = {");
 	pw_properties_serialize_dict(f, &data->capture_props->dict, 0);
 	fprintf(f, " } playback.props = {");
@@ -127,6 +119,7 @@ static int module_loopback_unload(struct module *module)
 
 	pw_properties_free(d->capture_props);
 	pw_properties_free(d->playback_props);
+	pw_properties_free(d->global_props);
 
 	return 0;
 }
@@ -134,17 +127,7 @@ static int module_loopback_unload(struct module *module)
 static const struct spa_dict_item module_loopback_info[] = {
 	{ PW_KEY_MODULE_AUTHOR, "Arun Raghavan <arun@asymptotic.io>" },
 	{ PW_KEY_MODULE_DESCRIPTION, "Loopback from source to sink" },
-	{ PW_KEY_MODULE_USAGE, "source=<source to connect to> "
-				"sink=<sink to connect to> "
-				"latency_msec=<latency in ms> "
-				"rate=<sample rate> "
-				"channels=<number of channels> "
-				"channel_map=<channel map> "
-				"sink_input_properties=<proplist> "
-				"source_output_properties=<proplist> "
-				"source_dont_move=<boolean> "
-				"sink_dont_move=<boolean> "
-				"remix=<remix channels?> " },
+	{ PW_KEY_MODULE_USAGE, pulse_module_options },
 	{ PW_KEY_MODULE_VERSION, PACKAGE_VERSION },
 };
 
@@ -152,16 +135,17 @@ static int module_loopback_prepare(struct module * const module)
 {
 	struct module_loopback_data * const d = module->user_data;
 	struct pw_properties * const props = module->props;
-	struct pw_properties *playback_props = NULL, *capture_props = NULL;
+	struct pw_properties *global_props = NULL, *playback_props = NULL, *capture_props = NULL;
 	const char *str;
 	struct spa_audio_info_raw info = { 0 };
 	int res;
 
 	PW_LOG_TOPIC_INIT(mod_topic);
 
+	global_props = pw_properties_new(NULL, NULL);
 	capture_props = pw_properties_new(NULL, NULL);
 	playback_props = pw_properties_new(NULL, NULL);
-	if (!capture_props || !playback_props) {
+	if (!global_props || !capture_props || !playback_props) {
 		res = -EINVAL;
 		goto out;
 	}
@@ -187,10 +171,12 @@ static int module_loopback_prepare(struct module * const module)
 		pw_properties_set(props, "sink", NULL);
 	}
 
-	if (module_args_to_audioinfo(module->impl, props, &info) < 0) {
+	if (module_args_to_audioinfo_keys(module->impl, props,
+				NULL, NULL, "channels", "channel_map", &info) < 0) {
 		res = -EINVAL;
 		goto out;
 	}
+	audioinfo_to_properties(&info, global_props);
 
 	if ((str = pw_properties_get(props, "source_dont_move")) != NULL) {
 		pw_properties_set(capture_props, PW_KEY_NODE_DONT_RECONNECT, str);
@@ -209,8 +195,13 @@ static int module_loopback_prepare(struct module * const module)
 		pw_properties_set(props, "remix", NULL);
 	}
 
-	if ((str = pw_properties_get(props, "latency_msec")) != NULL)
-		d->latency_msec = atoi(str);
+	if ((str = pw_properties_get(props, "latency_msec")) != NULL) {
+		uint32_t latency_msec = atoi(str);
+		char val[256];
+		pw_properties_setf(global_props, "target.delay.sec",
+				"%s", spa_json_format_float(val, sizeof(val),
+					latency_msec / 1000.0f));
+	}
 
 	if ((str = pw_properties_get(props, "sink_input_properties")) != NULL) {
 		module_args_add_props(playback_props, str);
@@ -223,12 +214,13 @@ static int module_loopback_prepare(struct module * const module)
 	}
 
 	d->module = module;
+	d->global_props = global_props;
 	d->capture_props = capture_props;
 	d->playback_props = playback_props;
-	d->info = info;
 
 	return 0;
 out:
+	pw_properties_free(global_props);
 	pw_properties_free(playback_props);
 	pw_properties_free(capture_props);
 
