@@ -49,7 +49,7 @@ struct data_loop {
 	struct pw_data_loop *impl;
 	bool autostart;
 	bool started;
-	int ref;
+	uint64_t last_used;
 };
 
 /** \cond */
@@ -204,20 +204,21 @@ static int setup_data_loops(struct impl *impl)
 	lib_name = pw_properties_get(this->properties, "context.data-loop." PW_KEY_LIBRARY_NAME_SYSTEM);
 
 	if ((str = pw_properties_get(this->properties, "context.data-loops")) != NULL) {
-		struct spa_json it[4];
+		struct spa_json it[2];
 		char key[512];
 		int r, len = strlen(str);
 		spa_autofree char *s = strndup(str, len);
 
 		i = 0;
-		spa_json_init(&it[0], s, len);
-		if (spa_json_enter_array(&it[0], &it[1]) < 0) {
+		if (spa_json_begin_array(&it[0], s, len) < 0) {
 			pw_log_error("context.data-loops is not an array in '%s'", str);
 			res = -EINVAL;
 			goto exit;
 		}
-		while ((r = spa_json_enter_object(&it[1], &it[2])) > 0) {
+		while ((r = spa_json_enter_object(&it[0], &it[1])) > 0) {
 			char *props = NULL;
+			const char *val;
+			int l;
 
 			if (i >= MAX_LOOPS) {
 				pw_log_warn("too many context.data-loops, using first %d",
@@ -229,17 +230,9 @@ static int setup_data_loops(struct impl *impl)
 			pw_properties_update(pr, &this->properties->dict);
 			pw_properties_set(pr, PW_KEY_LIBRARY_NAME_SYSTEM, lib_name);
 
-			while (spa_json_get_string(&it[2], key, sizeof(key)) > 0) {
-				const char *val;
-				int l;
-
-				if ((l = spa_json_next(&it[2], &val)) <= 0) {
-					pw_log_warn("malformed data-loop: key '%s' has no "
-							"value in '%.*s'", key, (int)len, str);
-					break;
-				}
+			while ((l = spa_json_object_next(&it[1], key, sizeof(key), &val)) > 0) {
 				if (spa_json_is_container(val, l))
-					l = spa_json_container_len(&it[2], val, l);
+					l = spa_json_container_len(&it[1], val, l);
 
 				props = (char*)val;
 				spa_json_parse_stringn(val, l, props, l+1);
@@ -587,6 +580,8 @@ void pw_context_destroy(struct pw_context *context)
 
 	if (context->work_queue)
 		pw_work_queue_destroy(context->work_queue);
+	if (context->timer_queue)
+		pw_timer_queue_destroy(context->timer_queue);
 
 	pw_properties_free(context->properties);
 	pw_properties_free(context->conf);
@@ -679,12 +674,12 @@ static struct pw_data_loop *acquire_data_loop(struct impl *impl, const char *nam
 			}
 		}
 
-		pw_log_debug("%d: name:'%s' class:'%s' score:%d ref:%d", i,
-				ln, l->impl->class, score, l->ref);
+		pw_log_debug("%d: name:'%s' class:'%s' score:%d last_used:%"PRIu64, i,
+				ln, l->impl->class, score, l->last_used);
 
 		if ((best_loop == NULL) ||
 		    (score > best_score) ||
-		    (score == best_score && l->ref < best_loop->ref)) {
+		    (score == best_score && l->last_used < best_loop->last_used)) {
 			best_loop = l;
 			best_score = score;
 		}
@@ -692,15 +687,15 @@ static struct pw_data_loop *acquire_data_loop(struct impl *impl, const char *nam
 	if (best_loop == NULL)
 		return NULL;
 
-	best_loop->ref++;
+	best_loop->last_used = get_time_ns(impl->this.main_loop->system);
 	if ((res = data_loop_start(impl, best_loop)) < 0) {
 		errno = -res;
 		return NULL;
 	}
 
-	pw_log_info("%p: using name:'%s' class:'%s' ref:%d", impl,
+	pw_log_info("%p: using name:'%s' class:'%s' last_used:%"PRIu64, impl,
 			best_loop->impl->loop->name,
-			best_loop->impl->class, best_loop->ref);
+			best_loop->impl->class, best_loop->last_used);
 
 	return best_loop->impl;
 }
@@ -744,9 +739,8 @@ void pw_context_release_loop(struct pw_context *context, struct pw_loop *loop)
 	for (i = 0; i < impl->n_data_loops; i++) {
 		struct data_loop *l = &impl->data_loops[i];
 		if (l->impl->loop == loop) {
-			l->ref--;
-			pw_log_info("release name:'%s' class:'%s' ref:%d", l->impl->loop->name,
-					l->impl->class, l->ref);
+			pw_log_info("release name:'%s' class:'%s' last_used:%"PRIu64,
+					l->impl->loop->name, l->impl->class, l->last_used);
 			return;
 		}
 	}
@@ -758,6 +752,14 @@ struct pw_work_queue *pw_context_get_work_queue(struct pw_context *context)
 	return context->work_queue;
 }
 
+SPA_EXPORT
+struct pw_timer_queue *pw_context_get_timer_queue(struct pw_context *context)
+{
+	if (context->timer_queue == NULL)
+		context->timer_queue =  pw_timer_queue_new(context->main_loop);
+	return context->timer_queue;
+}
+
 SPA_EXPORT
 struct pw_mempool *pw_context_get_mempool(struct pw_context *context)
 {
@@ -1527,8 +1529,8 @@ int pw_context_recalc_graph(struct pw_context *context, const char *reason)
 	struct pw_impl_node *n, *s, *target, *fallback;
 	const uint32_t *rates;
 	uint32_t max_quantum, min_quantum, def_quantum, rate_quantum, floor_quantum, ceil_quantum;
-	uint32_t n_rates, def_rate;
-	bool freewheel, global_force_rate, global_force_quantum, transport_start;
+	uint32_t n_rates, def_rate, transport;
+	bool freewheel, global_force_rate, global_force_quantum;
 	struct spa_list collect;
 
 	pw_log_info("%p: busy:%d reason:%s", context, impl->recalc, reason);
@@ -1541,7 +1543,6 @@ int pw_context_recalc_graph(struct pw_context *context, const char *reason)
 again:
 	impl->recalc = true;
 	freewheel = false;
-	transport_start = false;
 
 	/* clean up the flags first */
 	spa_list_for_each(n, &context->node_list, link) {
@@ -1774,10 +1775,16 @@ again:
 			/* Here we are allowed to change the rate of the driver.
 			 * Start with the default rate. If the desired rate is
 			 * allowed, switch to it */
-			target_rate = node_def_rate;
 			if (rate.denom != 0 && rate.num == 1)
-				target_rate = find_best_rate(node_rates, node_n_rates,
-						rate.denom, target_rate);
+				target_rate = rate.denom;
+			else
+				target_rate = node_def_rate;
+
+			target_rate = find_best_rate(node_rates, node_n_rates,
+						target_rate, node_def_rate);
+
+			pw_log_debug("%p: def_rate:%d target_rate:%d rate:%d/%d", context,
+					node_def_rate, target_rate, rate.num, rate.denom);
 		}
 
 		was_target_pending = n->target_pending;
@@ -1807,15 +1814,15 @@ again:
 
 		if (node_rate_quantum != 0 && current_rate != node_rate_quantum) {
 			/* the quantum values are scaled with the current rate */
-			node_def_quantum = node_def_quantum * current_rate / node_rate_quantum;
-			node_min_quantum = node_min_quantum * current_rate / node_rate_quantum;
-			node_max_quantum = node_max_quantum * current_rate / node_rate_quantum;
+			node_def_quantum = SPA_SCALE32(node_def_quantum, current_rate, node_rate_quantum);
+			node_min_quantum = SPA_SCALE32(node_min_quantum, current_rate, node_rate_quantum);
+			node_max_quantum = SPA_SCALE32(node_max_quantum, current_rate, node_rate_quantum);
 		}
 
 		/* calculate desired quantum. Don't limit to the max_latency when we are
 		 * going to force a quantum or rate and reconfigure the nodes. */
 		if (max_latency.denom != 0 && !force_quantum && !force_rate) {
-			uint32_t tmp = (max_latency.num * current_rate / max_latency.denom);
+			uint32_t tmp = SPA_SCALE32(max_latency.num, current_rate, max_latency.denom);
 			if (tmp < node_max_quantum)
 				node_max_quantum = tmp;
 		}
@@ -1832,7 +1839,7 @@ again:
 		else {
 			target_quantum = node_def_quantum;
 			if (latency.denom != 0)
-				target_quantum = (latency.num * current_rate / latency.denom);
+				target_quantum = SPA_SCALE32(latency.num, current_rate, latency.denom);
 			target_quantum = SPA_CLAMP(target_quantum, node_min_quantum, node_max_quantum);
 			target_quantum = SPA_CLAMP(target_quantum, floor_quantum, ceil_quantum);
 
@@ -1889,10 +1896,14 @@ again:
 				n->rt.position->clock.target_duration,
 				n->rt.position->clock.target_rate.denom, n->name);
 
+		transport = PW_NODE_ACTIVATION_COMMAND_NONE;
+
 		/* first change the node states of the followers to the new target */
 		spa_list_for_each(s, &n->follower_list, follower_link) {
-			if (s->transport)
-				transport_start = true;
+			if (s->transport != PW_NODE_ACTIVATION_COMMAND_NONE) {
+				transport = s->transport;
+				s->transport = PW_NODE_ACTIVATION_COMMAND_NONE;
+			}
 			if (s == n)
 				continue;
 			pw_log_debug("%p: follower %p: active:%d '%s'",
@@ -1900,10 +1911,10 @@ again:
 			ensure_state(s, running);
 		}
 
-		SPA_ATOMIC_STORE(n->rt.target.activation->command,
-				transport_start ?
-					PW_NODE_ACTIVATION_COMMAND_START :
-					PW_NODE_ACTIVATION_COMMAND_STOP);
+		if (transport != PW_NODE_ACTIVATION_COMMAND_NONE) {
+			pw_log_info("%s: transport %d", n->name, transport);
+			SPA_ATOMIC_STORE(n->rt.target.activation->command, transport);
+		}
 
 		/* now that all the followers are ready, start the driver */
 		ensure_state(n, running);
